'''
Created on Jun 14, 2011

@author: jklo
'''
import types
import sys
from .errors import SignatureException

class InvalidJSONError(ValueError):
    def __init__(self, msg):
        ValueError.__init__(self)
        self.message = msg


class PipeTool(object):

    def __init__(self):

        self.args = self.parseArgs()



    def run(self):
        from sign.Sign import Sign_0_21 as Signer
        from verify.Verify import Verify_0_21 as Verifier

        import json

        rawInput = self.readInput()
        envelopeList = self.parseInput(rawInput)

        if self.args.mode == "sign":
            self.signtool = Signer(privateKeyID=self.args.key,
                              passphrase=self.args.passphrase,
                              gnupgHome=self.args.gnupghome,
                              gpgbin=self.args.gpgbin, publicKeyLocations=self.args.key_location, sign_everything=False)

            is_test_data_opt = self.args.lr_test_data.lower() in ["true", "yes", "t", "y"]

            try:
                signedList = self.signEnvelopes(envelopeList, is_test_data=is_test_data_opt)
            except SignatureException as ex:
                sys.stderr.write("ERROR: "+ex.message+"\n")
                exit(1)

            if self.args.publish_url != None:
                self.publishEnvelopes(signedList)
            else:
                print json.dumps({ "documents": signedList })

        elif self.args.mode == "verify":
            self.verifytool = Verifier(gpgbin=self.args.gpgbin, gnupgHome=self.args.gnupghome)
            resultList = self.validateEnvelopes(envelopeList)
            print json.dumps({"results": resultList})

        elif self.args.mode == "publish":
            self.publishEnvelopes(envelopeList)




    def _set_test_key(self, envelope, remove=True):
        rmcount = 0

        if envelope.has_key("keys"):
            for item in envelope["keys"]:
                if item == "lr-test-data":
                    rmcount += 1
        if remove:
            while rmcount > 0:
                try:
                    envelope["keys"].remove("lr-test-data")
                except:
                    pass
                rmcount += -1
        elif not remove and rmcount == 0:
            envelope["keys"].append("lr-test-data")

        return envelope

    def _chunkList(self, fullList = [], chunkSize=10):
        numElements = len(fullList)
        for start in range(0, numElements, chunkSize):
            end = start+chunkSize
            if end > numElements:
                end = numElements
            yield fullList[start:end]


    def _validate_digital_signature(self, doc):
        from LRSignature import util
        from LRSignature import errors


        result = {}
        result = {"verified": False}
        if doc.has_key("doc_ID"):
            result["doc_ID"] = doc["doc_ID"]
        if doc.has_key("resource_locator"):
            result["resource_locator"] = doc["resource_locator"]

        try:
            result["verified"] = self.verifytool.verify(doc)

        except errors.MissingPublicKey:

            locations = doc['digital_signature']['key_location']
            numImported = 0
            for location in locations:
                rawKeys = util.fetchkeys(location)
                for rawKey in rawKeys:
                    numImported += util.storekey(self.sampleKey, gpgbin=self.args.gpgbin, gnupgHome=self.args.gnupghome)
                if numImported > 0:
                    break

            try:
                result["verified"] = self.verifytool.verify(doc)


            except errors.MissingPublicKey:
                result["verified"] = False
                result["error"] = "No available public key to validate."

        except errors.SignatureException as e:
            result["verified"] = False
            result["error"] = "Signature exception: "+e.message

        except Exception as e:
            result["verified"] = False
            result["error"] = e.message

        return result

    def validateEnvelopes(self, envelopes):
        result = []
        for envelope in envelopes:
            result.append(self._validate_digital_signature(envelope))
        return result


    def publishEnvelopes(self, envelopes):
        import urllib2,json
        req = urllib2.Request(self.args.publish_url, headers={"Content-type": "application/json; charset=utf-8"})
        if self.args.publish_username and self.args.publish_password:
            import base64
            base64string = base64.encodestring('%s:%s' % (self.args.publish_username, self.args.publish_password))[:-1]
            req.add_header("Authorization", "Basic %s" % base64string)
        status = []
        for chunk in self._chunkList(envelopes, self.args.publish_chunksize):
            res = urllib2.urlopen(req, data=json.dumps({ "documents":chunk }), timeout=self.args.publish_timeout)
            status.append(json.load(res))

        print json.dumps(status)

    def signEnvelopes(self, envelopes, is_test_data=True):
        signedEnvelopes = []
        for envelope in envelopes:
            self._set_test_key(envelope, remove=not is_test_data)
            signed = self.signtool.sign(envelope)
            signedEnvelopes.append(signed)
        return signedEnvelopes

    def parseArgs(self):
        import argparse, json
        parser = argparse.ArgumentParser(description='Sign files for Learning Registry')
        subparsers = parser.add_subparsers(help="sub-command help")

        sign_parser = subparsers.add_parser('sign')
        sign_parser.add_argument('--key', help='PGP Private Key ID', required=True)
        sign_parser.add_argument('--key-location', help='Location the PGP Public Key can be downloaded from', required=True, action="append")
        sign_parser.add_argument('--passphrase', help='Passphrase for PGP Private Key', default=None)
        sign_parser.add_argument('--lr-test-data', help='Publish as lr test data, default is True', default="True")
        sign_parser.add_argument('--publish-url', help='URL of publish service on node to send envelopes, default STDOUT', default=None)
        sign_parser.add_argument('--publish-chunksize', help='publish chunksize, default 25', type=int, default=25)
        sign_parser.add_argument('--publish-timeout', help='publish timeout in seconds, default 300', type=int, default=300)
        sign_parser.add_argument('--publish-username', help='publish userame for basic HTTP auth', default=None)
        sign_parser.add_argument('--publish-password', help='publish password for basic HTTP auth', default=None)
        sign_parser.add_argument('--gpgbin', help='Path to GPG binary')
        sign_parser.add_argument('--gnupghome', help='Path to GPG home directory')
        sign_parser.set_defaults(mode="sign")

        verify_parser = subparsers.add_parser('verify')
        verify_parser.set_defaults(mode="verify")
        verify_parser.add_argument('--gpgbin', help='Path to GPG binary')
        verify_parser.add_argument('--gnupghome', help='Path to GPG home directory')

        publish_parser = subparsers.add_parser('publish')
        publish_parser.set_defaults(mode="publish")
        publish_parser.add_argument('--lr-test-data', help='Publish as lr test data, default is True', default="True")
        publish_parser.add_argument('--publish-url', help='URL of publish service on node to send envelopes, default STDOUT', default=None)
        publish_parser.add_argument('--publish-chunksize', help='publish chunksize, default 25', type=int, default=25)
        publish_parser.add_argument('--publish-timeout', help='publish timeout in seconds, default 300', type=int, default=300)
        publish_parser.add_argument('--publish-username', help='publish userame for basic HTTP auth', default=None)
        publish_parser.add_argument('--publish-password', help='publish password for basic HTTP auth', default=None)
        publish_parser.add_argument('--gpgbin', help='Path to GPG binary')
        publish_parser.add_argument('--gnupghome', help='Path to GPG home directory')

        parser.add_argument('--gpgbin', help='Path to GPG binary', default="gpg")
        parser.add_argument('--gnupghome', help='Path to GPG home directory', default="~/.gnupg")
    #        parser.add_argument('--config', help='JSON Configuration file', default=None, type=argparse.FileType('r'))
        args = parser.parse_args()#

        return args


    def parseInput(self, input=None):
        import json

        def getHarvestRecords(obj):
            def harvestGetRecordGenerator(results):
                for item in results:
                    if "resource_data" in item:
                        yield item["resource_data"]

            def harvestListRecordsGenerator(results):
                for item in results:
                    if "record" in item and "resource_data" in item["record"]:
                        yield item["record"]["resource_data"]

            try:
                root = obj["getrecord"]["record"]
                return harvestGetRecordGenerator(root)
            except:
                pass

            try:
                root = obj["listrecords"]
                return harvestListRecordsGenerator(root)
            except:
                pass

            return None


        if input is not None:
            try:
                jsobject = json.loads(input)

                records = getHarvestRecords(jsobject)
                if records != None:
                    return records
                if isinstance(jsobject, types.DictionaryType):
                    if jsobject.has_key("documents") and isinstance(jsobject["documents"], types.ListType):
                        return jsobject["documents"]
                    return [jsobject]
                elif isinstance(jsobject, types.ListType):
                    return jsobject

            except Exception, e:
                raise InvalidJSONError(e.message)
        return None



    def readInput(self):
        import json
        pipedInput = ''
        try:
            while True:
                pipedInput += raw_input()

        except EOFError:
            pass

        return pipedInput


if __name__ == "__main__":
    tool = PipeTool()
    tool.run()



'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on May 6, 2011

@author: jklo
'''


MISSING_SIGNATURE = "MISSING_SIGNATURE"
MISSING_KEY_LOCATION = "MISSING_KEY_LOCATION"
BAD_KEY_OWNER = "BAD_KEY_OWNER"

class SignatureException(Exception):
    def __init__(self, message=None):
        Exception.__init__(self)
        self.message = message


class UnknownKeyException(SignatureException):
    '''
    Exception to be thrown when a key is not found in local keyring.
    '''

    def __init__(self, keyid):
        SignatureException.__init__(self)
        self.keyid = keyid


class UnsupportedSignatureAlgorithm(SignatureException):
    '''
    Exception to be thrown when an unsupported method of signing is encountered.

    Params:
        alg : the algorithm encountered
    '''

    def __init__(self, alg=None):
        '''
        Constructor
        '''
        Exception.__init__(self)
        self.alg = alg


class BadSignatureFormat(SignatureException):
    '''
    Exception to be thrown when a supported method of signing is advertised but not adhered.
    '''
    pass

class MissingPublicKey(SignatureException):
    '''
    Exception to be raised when the public key cannot be found.
    '''
    def __init__(self, message=None, keyid=None):
        Exception.__init__(self)
        self.message = message
        self.keyid = keyid


class InvalidPassphrase(SignatureException):
    '''
    Exception to be raised when an invalid passphrase is provided for key
    '''
    pass

class IncompatibleDocumentVersion(SignatureException):
    '''
    Exception to be raised when a document does not have a compatible version for the signer
    '''
    pass



'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on May 9, 2011

@author: jklo
'''
from gnupg import GPG
from ..sign.Sign import *
from ..errors import *
import types, re, copy, os, sys
import cStringIO

from abc import ABCMeta


class VerifyBase(SignBase):
    __metaclass__ = ABCMeta
    '''
    Base class for verifying signatures
    '''

    def __init__(self, gpgbin="/usr/local/bin/gpg", gnupgHome=os.path.expanduser(os.path.join("~", ".gnupg"))):
        '''
        Constructor
        '''
        self.gnupgHome = gnupgHome
        self.gpgbin = gpgbin
        SignBase.__init__(self, privateKeyID=None, passphrase=None, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=[])


    def _getSignatureInfo(self, envelope={}):
            sigInfo = None

            if envelope.has_key("digital_signature"):
                sigInfo = envelope["digital_signature"]
                if sigInfo.has_key("signing_method"):
                    if sigInfo["signing_method"] == self.signatureMethod:
                        if not sigInfo.has_key("signature") or sigInfo["signature"] == None or len(sigInfo["signature"]) == 0:
                            raise BadSignatureFormat(MISSING_SIGNATURE)
                        elif not (sigInfo.has_key("key_location") and isinstance(sigInfo["key_location"], types.ListType) and len(sigInfo["key_location"]) > 0 ):
                            raise BadSignatureFormat(MISSING_KEY_LOCATION)
                        elif sigInfo.has_key("key_owner") and not isinstance(sigInfo["key_owner"], types.StringTypes):
                            raise BadSignatureFormat(BAD_KEY_OWNER)
                    else:
                        raise UnsupportedSignatureAlgorithm(sigInfo["signing_method"])
                else:
                    raise UnsupportedSignatureAlgorithm(None)


            return sigInfo

    def _extractHashFromSignature(self, signatureBlock=""):

        def removeHead(mesg=[]):
            status = 0
            mcopy = copy.deepcopy(mesg)
            for line in mesg:
                if re.match("^-----BEGIN PGP (SIGNED ){0,1}MESSAGE-----$", line) != None:
                    status = 1
                elif (status == 1 or status == 2) and re.match("^[^:]+: .+$", line) != None:
                    status = 2
                elif (status == 1 or status == 2) and line == "":
                    status = 3

                mcopy.pop(0)

                if status == 3:
                    break

            return mcopy

        def removeTail(mesg=[]):
            msgOnly = ""
            for line in mesg:
                if re.match("^-----BEGIN PGP SIGNATURE-----$", line):
                    break
                msgOnly += line
            return msgOnly

        if sys.version_info > (2, 7):
            sig = re.split("\r\n|\r|\n", signatureBlock, flags=re.MULTILINE)
        else:
            sig = re.split("\r\n|\r|\n", signatureBlock)

        hash = removeTail(removeHead(sig))

        return hash

    def get_and_verify(self, envelope):
        '''
        Get the OpenPGP validation info and Verify integrity of the provided envelope.

        Returns:
            None if no signature block exists
            gnupg.Verify object if signature & integrity check pass
        Raises:
            BadSignatureFormat if signature & integrity check do not pass
            MissingPublicKey if public key for signed document is missing
        '''

        sigInfo = self._getSignatureInfo(envelope)

        if sigInfo != None:

            verified = self.gpg.verify(sigInfo["signature"])
            if verified.valid == True:
                verifiedHash = self._extractHashFromSignature(sigInfo["signature"])

                if self.get_message(envelope) == verifiedHash:
                    return verified
                else:
                    raise BadSignatureFormat("valid signature, envelope hash bad match.")
            elif verified.valid == False and verified.status == 'no public key':
                raise MissingPublicKey(message=verified.data, keyid=verified.key_id)
            else:
                raise BadSignatureFormat("invalid signature")
        return None

    def verify(self, envelope):
        '''
        Verify integrity of the provided envelope.

        Returns:
            None if no signature block exists
            True if signature & integrity check pass
            False if signature & integrity check do not pass

        Raises:
            MissingPublicKey if public key for signed document is missing
        '''

        sigInfo = self._getSignatureInfo(envelope)

        if sigInfo != None:

            verified = self.gpg.verify(sigInfo["signature"])
            if verified.valid == True:
                verifiedHash = self._extractHashFromSignature(sigInfo["signature"])

                if self.get_message(envelope) == verifiedHash:
                    return True
                else:
                    return False
            elif verified.valid == False and verified.status == 'no public key':
                raise MissingPublicKey(message=verified.data, keyid=verified.key_id)
            else:
                return False
        return None


class Verify_0_21(Sign_0_21, VerifyBase):
    pass

class Verify_0_23(Sign_0_23, VerifyBase):
    pass

class Verify_0_49(Sign_0_49, VerifyBase):
    pass

class Verify_0_51(Sign_0_51, VerifyBase):
    pass


if __name__ == "__main__":
    verify = Verify_0_21()
    pass




'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on May 11, 2011

@author: jklo
'''
import unittest, os, errno
import gnupg
from LRSignature import util as util

class Test(unittest.TestCase):
    '''Unit test cases for testing utility methods'''
    
    def mkdir_p(self, path):
        try:
            os.makedirs(path)
        except OSError as exc: # Python >2.5
            if exc.errno == errno.EEXIST:
                pass
            else: raise
    def rm_rf(self, path):
        for r,d,f in os.walk(path):
            for files in f:
                os.remove ( os.path.join(r,files) )
            os.removedirs( r ) 
            
    def chmod(self, path, mode):
        try:
            os.chmod(path, mode)
        except:
            pass

    def setUp(self):
        self.gnupghome =  os.path.expanduser(os.path.abspath(os.path.join("..", "gnupg_home")))
        self.gpgbin = 'gpg'
        
        self.rm_rf(self.gnupghome)
        self.mkdir_p(self.gnupghome)
        
        self.gpg = gnupg.GPG(gnupghome=self.gnupghome, gpgbinary=self.gpgbin)
        
        self.sampleKeyId="A8A790EA220403B7"
        self.sampleKeyFingerprint="E8BCBCB8994A293C10DDCBC5A8A790EA220403B7"
        self.sampleKey = '''-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: PGP Universal 2.9.1 (Build 347)
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=aCWc
-----END PGP PUBLIC KEY BLOCK-----'''
        


    def tearDown(self):
        self.rm_rf(self.gnupghome)
        pass

    def checkKey(self, key, keysource=""):
        '''Utility method to check to see if fetched key is valid by importing into GPG keyring'''
        keys = self.gpg.list_keys(secret=False)
        numkeys = len(keys)
        
        result = self.gpg.import_keys(key)
        assert result != None and result.imported == 1, "GPG import does not indicate key import occurred for {0}.".format(keysource)
        
        keys = self.gpg.list_keys(secret=False)
        assert len(keys) == numkeys+1, "Imported key did not get added to keyring for {0}.".format(keysource)
        
    def testPGPFetchKey(self):
        '''Attempt to retrieve key via HTTP GET from Symantec's PGP keyserver'''
        keyURL = "http://keyserver.pgp.com/vkd/DownloadKey.event?keyid=0xA8A790EA220403B7"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 1, "key could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="PGP sourced key")
        
        
    def testSKSFetchKey(self):
        '''Attempt to retrieve key via HTTP GET from SKS keyserver'''
        keyURL = "http://pool.sks-keyservers.net:11371/pks/lookup?op=get&search=0xA8A790EA220403B7"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 1, "key could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="SKS source key")
        
    def testWebSiteFetchKey(self):
        '''Attempt to retrieve key via HTTP GET that is a part of an HTML web page'''
        keyURL = "http://sites.google.com/site/learningregistrytestdata/home/single-public-pgp-key"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 1,  "key could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="HTML Web Page sourced key")
        
    def testHTTPFetchKey(self):
        '''Attempt to retrieve key via HTTP GET that is a part of a file download request'''
        keyURL = "http://sites.google.com/site/learningregistrytestdata/home/single-public-pgp-key/key0xA8A790EA220403B7.asc?attredirects=0&d=1"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 1, "key could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="HTTP Download sourced key")
        
    def testMultipleFetchKey(self):
        '''Attempt to retrieve multiple keys via HTTP GET that is a part of a file download request'''
        keyURL = "http://sites.google.com/site/learningregistrytestdata/home/multiple-public-pgp-keys"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 2, "keys could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="HTTP Download sourced key 1")
        self.checkKey(fetchedKeys[1], keysource="HTTP Download sourced key 2")
        
    def testGoogleDocFetchKey(self):
        '''Attempt to retrieve key via HTTP file download via Google Docs'''
        keyURL = "https://docs.google.com/uc?id=0ByJYdR0YE41yNDg2ZTkzYTItNmJjNC00YTgwLTkyYzctZTNiMzAwYzhjYTdh&export=download&hl=en"
        fetchedKeys = util.fetchkeys(keyURL)
        assert len(fetchedKeys) == 1, "key could not be retrieved from URL"
        
        self.checkKey(fetchedKeys[0], keysource="Google Doc source key")

    
    def testStoreKey(self):
        '''Use a known ASCII Armored key and store it via util.storekey(...)'''
        
        keys = self.gpg.list_keys(secret=False)
        for key in keys:
            assert key['keyid'] != self.sampleKeyId, "Imported sample key already in keyring"
        
        numkeys = len(keys)
        
        numImported = util.storekey(self.sampleKey, gnupghome=self.gnupghome, gpgbin=self.gpgbin)
        
        assert numImported == 1, "sample key not imported into keyring"
        
        keys = self.gpg.list_keys(secret=False)
        
        assert len(keys) == 1, "no key found matching id"
        for key in keys:
            assert key['keyid'] == self.sampleKeyId, "exported key is not expected"
            
            
    
    def testStoreExistingKey(self):
        '''Use a known ASCII Armored key and try storing it twice.  It should not get imported a second time'''
        
        keys = self.gpg.list_keys(secret=False)
        for key in keys:
            assert key['keyid'] != self.sampleKeyId, "Imported sample key already in keyring"
        
        numkeys = len(keys)
        
        numImported = util.storekey(self.sampleKey, gnupghome=self.gnupghome, gpgbin=self.gpgbin)
        
        assert numImported == 1, "sample key not imported into keyring"
        
        numImported = util.storekey(self.sampleKey, gnupghome=self.gnupghome, gpgbin=self.gpgbin)
        
        assert numImported == 0, "sample key imported when it shouldn't have been into keyring"
        
        keys = self.gpg.list_keys(secret=False)
        
        assert len(keys) == 1, "no key found matching id"
        for key in keys:
            assert key['keyid'] == self.sampleKeyId, "exported key is not expected"
    

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testFetchKey']
    unittest.main()


'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on Apr 27, 2011

@author: jklo
'''

import urllib2
import re
import gnupg
import os


def fetchkeys(url):
    '''
    Fetches one or more PGP public key from URL
    '''

    req = urllib2.Request(url)
    res = urllib2.urlopen(req)
    
    response = res.read()
    
    obj = re.finditer("((?!<.*)(-----BEGIN PGP PUBLIC KEY BLOCK-----(\\r)?\\n[^-]+-----END PGP PUBLIC KEY BLOCK-----))+", response, re.MULTILINE | re.DOTALL)
    
    rawKeys = []
    for o in obj:
        rawKeys.append(response[o.start():o.end()])
    return rawKeys

def storekey(keydata, gnupghome=os.path.expanduser(os.path.join("~", ".gnupg")), gpgbin="/usr/local/bin/gpg"):
    gpg = gnupg.GPG(gpgbinary=gpgbin, gnupghome=gnupghome)
    
    result = gpg.import_keys(keydata)
    return result.imported


if __name__ == "__main__":
    keyURLs = ["http://pool.sks-keyservers.net:11371/pks/lookup?op=get&search=0xA8A790EA220403B7",
              "http://sites.google.com/site/learningregistrytestdata/home/multiple-public-pgp-keys"]
    for url in keyURLs:
        
        keys = fetchkeys(url)
        print "Key count: {0}; URL:{1}".format(len(keys),url)
        for key in keys:
            print key
    


'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on Apr 27, 2011

@author: jklo
'''
import re
import hashlib
import gnupg
import types
import os, copy
from ..errors import UnknownKeyException, InvalidPassphrase, IncompatibleDocumentVersion
from ..bencode import bencode
from abc import ABCMeta

def _cmp_version(version1, version2):
    def normalize(v):
        return [int(x) for x in re.sub(r'(\.0+)*$','', v).split(".")]
    return cmp(normalize(version1), normalize(version2))

class SignBase(object):
    __meta_class__ = ABCMeta
    signatureMethod = "LR-PGP.1.0"
    min_doc_version = "0.21.0"


    '''
    Base class for signing LR envelopes following version 0.21.0 of the LR Specification:
        see: https://docs.google.com/document/d/191BTary350To_4JokBUFZLFRMOEfGYrl_EHE6QZxUr8/edit
    '''

    def __init__(self, privateKeyID=None, passphrase=None, gnupgHome=os.path.expanduser(os.path.join("~", ".gnupg")), gpgbin="/usr/local/bin/gpg", publicKeyLocations=[], sign_everything=True):
        '''
        Constructor
        '''
        self.privateKeyID = privateKeyID
        self.passphrase = passphrase
        self.gnupgHome = gnupgHome
        self.gpgbin = gpgbin
        self.publicKeyLocations = publicKeyLocations
        self.sign_everything = sign_everything

        self.gpg = gnupg.GPG(gnupghome=self.gnupgHome, gpgbinary=self.gpgbin)

        secretKeys = self.gpg.list_keys(secret=True)

        if self.privateKeyID != None:
            # remove spaces from key as `gpg --fingerprint` returns a key with spaces
            self.privateKeyID = self.privateKeyID.replace(' ', '')

            privateKeyAvailable = False
            for skey in secretKeys:
                if skey["keyid"] == self.privateKeyID:
                    privateKeyAvailable = True
                    self.privateKeyInfo = skey
                    break
                if skey["fingerprint"] == self.privateKeyID:
                    privateKeyAvailable = True
                    self.privateKeyInfo = skey

            if privateKeyAvailable == False:
                raise UnknownKeyException(self.privateKeyID)

    def _version_check(self, doc):
        if _cmp_version(doc["doc_version"], self.min_doc_version) < 0:
            raise IncompatibleDocumentVersion(
                "Document version of %s found, looking for %s or greater" % (doc["doc_version"], self.min_doc_version)
            )

        return True

    def _bnormal(self, obj = {}):
            if isinstance(obj, types.NoneType):
                return "null"
            # Boolean needs to be checked before numeric types as Booleans are also IntType
            elif isinstance(obj, types.BooleanType):
                return str(obj).lower()
            elif isinstance(obj, (types.FloatType, types.IntType, types.LongType, types.ComplexType)):
                print "Dropping number: {0}\n".format(obj)
                raise TypeError("Numbers not permitted")
                # return str(obj)
            elif isinstance(obj, types.StringType):
                return obj
            elif isinstance(obj, types.UnicodeType):
                return obj
            elif isinstance(obj, types.ListType):
                nobj = []
                for child in obj:
                    try:
                        nobj.append(self._bnormal(child))
                    except TypeError:
                        pass
                return nobj
            elif isinstance(obj, types.DictType):
                for key in obj.keys():
                    try:
                        obj[key] = self._bnormal(obj[key])
                    except TypeError:
                        obj[key] = self._bnormal(None)
                return obj
            else:
                return obj

    def _stripEnvelope(self, envelope={}):
        fields = ["digital_signature", "publishing_node", "update_timestamp", "node_timestamp", "create_timestamp", "doc_ID", "_id", "_rev"]
        sigObj = copy.deepcopy(envelope)
        for field in fields:
            if sigObj.has_key(field):
                del sigObj[field]
        return sigObj


    def _buildCanonicalString(self, envelope = {}):
        encoded = bencode(envelope)
        return encoded

    def _hash(self, msg):
        hashedDigest = hashlib.sha256(msg.encode("utf-8")).hexdigest()
        return hashedDigest

    def get_message(self, envelope):
        '''
        Hashes the contents of an LR envelope in the following manner:

            1. remove all fields from envelope except for the following:
                "doc_type", "doc_version", "resource_data_type", "active", "submitter_type", "submitter",
                "submitter_timestamp", "submitter_TTL",  "submission_TOS", "submission_attribution",
                "resource_locator", "keys", "resource_TTL", "payload_placement"
            2. Bencode the remaining envelope [http://en.wikipedia.org/wiki/Bencode]
            3. Hash the Bencoded string using a SHA256
            4. Convert SHA256 to hexadecimal digest.

        Returns digest as string.
        '''

        stripped = self._stripEnvelope(envelope)
        normalized = self._bnormal(stripped)
        canonical = self._buildCanonicalString(normalized)
        hashedDigest = self._hash(canonical)

        return hashedDigest

    def _get_privatekey_owner(self):
        if self.privateKeyInfo.has_key("uids") and isinstance(self.privateKeyInfo["uids"], types.ListType):
            return ", ".join(self.privateKeyInfo["uids"])

        return None


    def _get_sig_block(self, sigdata):
        signature = {
            "signature": sigdata,
            "key_location": self.publicKeyLocations,
            "signing_method": self.signatureMethod
        }

        pkowner = self._get_privatekey_owner()
        if pkowner != None:
            signature["key_owner"] = pkowner

        return signature

    def sign(self, envelope):
        '''
        Hashes and Signs a LR envelope according to the version 2.0 LR Specification
        '''

        # check version compatibility
        try:
            self._version_check(envelope)
        except IncompatibleDocumentVersion as ex:
            # raise IncompatibleDocumentVersion
            if not self.sign_everything:
                raise ex

        msg = self.get_message(envelope)

        signPrefs = {
                     "keyid": self.privateKeyID,
                     "passphrase": self.passphrase,
                     "clearsign": True
                }

        result = self.gpg.sign(msg, **signPrefs)

        if result.data == '':
            raise InvalidPassphrase('The passphrase provided is incorrect')


        envelope["digital_signature"] = self._get_sig_block(result.data)

        return envelope



# Alias classes to align with specification versions

class Sign_0_21(SignBase):
    min_doc_version = "0.21.0"

class Sign_0_23(SignBase):
    min_doc_version = "0.21.0"

class Sign_0_49(SignBase):
    min_doc_version = "0.49.0"

class Sign_0_51(SignBase):
    min_doc_version = "0.51.0"


if __name__ == "__main__":

    sign = Sign_0_21("C37C805D164B052C", passphrase="2dimples")




'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on May 6, 2011

@author: jklo
'''
import unittest, json, calendar, time, os, logging
from gnupg import GPG
from LRSignature.sign.Sign import Sign_0_21, Sign_0_23, Sign_0_51
from LRSignature.errors import UnknownKeyException
import types
import sys

log =  logging.getLogger(__name__)

class Test(unittest.TestCase):
    '''Unit test cases for signing envelopes from the Learning Registry'''
    def __init__(self, methodName="runTest"):

        self.gpgbin="gpg"
        self.gnupgHome = os.path.expanduser(os.path.abspath(os.path.join("..", "gnupg_home")))

        try:
            os.makedirs(self.gnupghome)
        except:
            pass

        self.gpg = None

        unittest.TestCase.__init__(self, methodName)

        self.testDataDir = None
        self.testDataUnicode = None

        configFile = os.path.join("config.cfg")
        if os.path.exists(configFile):
            config = json.load(file(configFile))

            if config.has_key("global"):
                if config["global"].has_key("testdata") and os.path.exists(config["global"]["testdata"]):
                    self.testDataDir = config["global"]["testdata"]

                if config["global"].has_key("testdata_unicode") and os.path.exists(config["global"]["testdata_unicode"]):
                    self.testDataUnicode = config["global"]["testdata_unicode"]



    def setUp(self):
        now = time.localtime()
        now = calendar.timegm(now)
        self.goodEmail = "signTest-{0}@learningregistry.org".format(now)
        self.goodRealName = "Autogenerated Sign Test"
        self.goodpassphrase = "supersecret"


        try:
            for root, dirs, files in os.walk(self.gnupgHome):
                for filename in files:
                    try:
                        os.unlink(os.path.join(root, filename))
                    except:
                        pass
                os.removedirs( root )
        except:
            pass

        os.makedirs(self.gnupgHome)


        self.gpg = GPG(gpgbinary=self.gpgbin, gnupghome=self.gnupgHome)

        input = self.gpg.gen_key_input(name_real=self.goodRealName, name_email=self.goodEmail, passphrase=self.goodpassphrase)
        self.goodPrivateKey = self.gpg.gen_key(input)

        privateKeyAvailable = False
        privateKeys = self.gpg.list_keys(secret=True)
        for skey in privateKeys:
            if skey["keyid"] == self.goodPrivateKey.fingerprint:
                privateKeyAvailable = True
                self.privateKeyInfo = skey
                break
            if skey["fingerprint"] == self.goodPrivateKey.fingerprint:
                privateKeyAvailable = True
                self.privateKeyInfo = skey
                break

        assert privateKeyAvailable == True, "Could not locate generated Private Key"

        self.goodkeyid = self.privateKeyInfo["keyid"]

        self.goodowner = self.privateKeyInfo["uids"][0]


        self.badkeyid = "XXXXXXXXXXXXXXXX"
        self.badpassphrase = "bad passphrase"

        self.sampleJSON = '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data":"<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ",
                "keys":["United States--Description and travel.","eng"],
                "submitter_type":"agent",
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "submitter":"NSDL 2 LR Data Pump",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.10.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            '''

        self.sampleJSON__0_23_0 = '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data":"<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ",
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            '''

        self.sampleJSON__0_51_0 = '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2014-04-01 13:36:04.617999",
                "resource_data":"<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ",
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2014-04-01 13:32:04.617999",
                "doc_version":"0.51.0",
                "create_timestamp":"2014-04-01 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            '''


        self.sampleJSON_no_coercion = '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data": {
                    "name": "Test coersion",
                    "nullable": null,
                    "booleanT": true,
                    "booleanF": false
                },
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            ''';


        self.sampleJSON_lossy_sha256 = [
            '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data": {
                    "name": "Test Lossy"
                    "integer": 1
                },
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            ''',
            '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data": {
                    "name": "Test Lossy"
                    "float": 1.1
                },
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            ''',
            '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data": {
                    "name": "Test Lossy"
                    "max_int": 9007199254740990
                },
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            ''',
            '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data": {
                    "name": "Test Lossy"
                    "min_int": -9007199254740990
                },
                "keys":["United States--Description and travel.","eng"],
                "identity":{
                    "submitter_type":"agent",
                    "submitter":"NSDL 2 LR Data Pump"
                },
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.23.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            '''

        ]

        self.sampleJSON_strip = '''{"keys": ["United States--Description and travel.", "eng"], "TOS": {"submission_attribution": "unidentified", "submission_TOS": "http://example.com/tos/unknown"}, "payload_placement": "inline", "active": true, "resource_locator": "http://hdl.loc.gov/loc.gdc/lhbtn.12281", "doc_type": "resource_data", "resource_data": "<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ", "submitter_type": "agent", "resource_data_type": "metadata", "payload_schema_locator": "http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd", "payload_schema": ["oai_dc"], "doc_version": "0.10.0", "submitter": "NSDL 2 LR Data Pump"}'''
        self.sampleJSON_strip_normal = '''{"keys": ["United States--Description and travel.", "eng"], "TOS": {"submission_attribution": "unidentified", "submission_TOS": "http://example.com/tos/unknown"}, "payload_placement": "inline", "active": "true", "resource_locator": "http://hdl.loc.gov/loc.gdc/lhbtn.12281", "doc_type": "resource_data", "resource_data": "<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ", "submitter_type": "agent", "resource_data_type": "metadata", "payload_schema_locator": "http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd", "payload_schema": ["oai_dc"], "doc_version": "0.10.0", "submitter": "NSDL 2 LR Data Pump"}'''
        self.sampleJSON_strip_normal_bencode = '''d3:TOSd14:submission_TOS30:http://example.com/tos/unknown22:submission_attribution12:unidentifiede6:active4:true8:doc_type13:resource_data11:doc_version6:0.10.04:keysl38:United States--Description and travel.3:enge17:payload_placement6:inline14:payload_schemal6:oai_dce22:payload_schema_locator90:http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd13:resource_data968:<oai_dc:dc xmlns:oai_dc="http://www.openarchives.org/OAI/2.0/oai_dc/" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.openarchives.org/OAI/2.0/" xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd">\n<dc:title>A chat about America. October and November, 1884.</dc:title>\n<dc:creator>J. P.</dc:creator>\n<dc:subject>United States--Description and travel.</dc:subject>\n<dc:description>"Printed for private circulation only."</dc:description>\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\n<dc:date>1885</dc:date>\n<dc:type>text</dc:type>\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\n<dc:language>eng</dc:language>\n<dc:coverage>United States</dc:coverage>\n</oai_dc:dc>\n      18:resource_data_type8:metadata16:resource_locator38:http://hdl.loc.gov/loc.gdc/lhbtn.122819:submitter19:NSDL 2 LR Data Pump14:submitter_type5:agente'''
        self.sampleJSON_sha256 = '''ef1b3b63adc663602c7a3c7595951b2761b34f5f6490ea1acee3df0fd97db03c'''

        self.sampleKeyLocations = [
                                   "http://example.com/mykey",
                                   "http://example2.com/mykey"
                                   ]


        self.signatureTemplate = '{{"key_location": [{0}], "key_owner": "'+self.goodowner+'", "signing_method": "LR-PGP.1.0", "signature": "{1}"}}'

    def tearDown(self):
        self.gpg.delete_keys([self.goodPrivateKey.fingerprint, ], secret=True)
        self.gpg.delete_keys([self.goodPrivateKey.fingerprint, ], secret=False)
        pass

    def testMissingPrivateKey(self):

        def missingKey():
            try:
                sign = Sign_0_21(self.badkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
            except UnknownKeyException as e:
                assert e.keyid == self.badkeyid, "keyid in exception doesn't match key passed to sign."
                raise e

        self.assertRaises(UnknownKeyException, missingKey)

    def testPresentPrivateKey(self):
        sign = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        assert sign.privateKeyID == self.goodkeyid

    def testStrip(self):
        origJson = json.loads(self.sampleJSON)
        benchmark = json.loads(self.sampleJSON_strip)

        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        stripJson = signer._stripEnvelope(origJson)
        assert benchmark == stripJson

    def testStripNormal(self):
        origJson = json.loads(self.sampleJSON)
        benchmark = json.loads(self.sampleJSON_strip_normal)

        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        stripJson = signer._stripEnvelope(origJson)
        normalJson = signer._bnormal(stripJson)
        assert benchmark == normalJson

    def testStripNormalBencode(self):
        origJson = json.loads(self.sampleJSON)
        benchmark = self.sampleJSON_strip_normal_bencode

        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        stripJson = signer._stripEnvelope(origJson)
        normalJson = signer._bnormal(stripJson)
        bencodeJson = signer._buildCanonicalString(normalJson)
        assert benchmark == bencodeJson

    def testStripNormalBencodeHash(self):
        origJson = json.loads(self.sampleJSON)
        benchmark = self.sampleJSON_sha256

        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        stripJson = signer._stripEnvelope(origJson)
        normalJson = signer._bnormal(stripJson)
        bencodeJson = signer._buildCanonicalString(normalJson)
        hashed = signer._hash(bencodeJson)
        assert benchmark == hashed

    def testKnownLossyHash(self):
        for idx, lossyJSON in enumerate(self.sampleJSON_lossy_sha256):
            origJson = json.loads(self.sampleJSON)

            signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
            stripJson = signer._stripEnvelope(origJson)
            normalJson = signer._bnormal(stripJson)
            bencodeJson = signer._buildCanonicalString(normalJson)
            hashed = signer._hash(bencodeJson)

            if idx > 0:
                assert prev_hashed == hashed, "Hashes should match - algorithm is lossy"
                assert prev_bencodeJson == bencodeJson, "Bencode strings should match - algorithm is lossy"

            prev_bencodeJson = bencodeJson
            prev_hashed = hashed

    def testGetMessage(self):
        origJson = json.loads(self.sampleJSON)
        benchmark = self.sampleJSON_sha256

        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        message = signer.get_message(origJson)
        assert benchmark == message

    def testPrivateKeyOwner(self):
        benchmark = self.goodowner
        signer = Sign_0_21(self.goodkeyid, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        assert benchmark == signer._get_privatekey_owner()

    def testSigBlock(self):
        origJson = json.loads(self.sampleJSON)
        arbitrarySigdata = "ABCDEF0123456789-abcdef"
        arbitraryKeyLoc = self.sampleKeyLocations

        keyloc = ",".join(map(lambda x: '"{0}"'.format(x), arbitraryKeyLoc))
        benchmark = json.loads(self.signatureTemplate.format(keyloc, arbitrarySigdata))

        signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        assert benchmark == signer._get_sig_block(arbitrarySigdata)

    def testSignNoEnvelopeCoercion(self):
        origJson = json.loads(self.sampleJSON_no_coercion)
        arbitraryKeyLoc = self.sampleKeyLocations

        assert origJson["resource_data"]["nullable"] == None, "Expected null"
        assert origJson["resource_data"]["booleanT"] == True, "Expected true"
        assert origJson["resource_data"]["booleanF"] == False, "Expected false"

        signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert signed.has_key("digital_signature")
        assert origJson["resource_data"]["nullable"] == None, "Expected null"
        assert origJson["resource_data"]["booleanT"] == True, "Expected true"
        assert origJson["resource_data"]["booleanF"] == False, "Expected false"

    def testSign_0_10__no_passthru(self):
        origJson = json.loads(self.sampleJSON)
        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert signed.has_key("digital_signature")

        sig = signed["digital_signature"]
        assert sig.has_key("signature")
        assert sig["signature"] != None and len(sig["signature"]) > 0

    def test_Sign__0_10__passthru(self):
        origJson = json.loads(self.sampleJSON)
        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, sign_everything=False, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert not signed.has_key("digital_signature")

        # sig = signed["digital_signature"]
        # assert sig.has_key("signature")
        # assert sig["signature"] != None and len(sig["signature"]) > 0

    def test_Sign__0_23__passthru(self):
        origJson = json.loads(self.sampleJSON__0_23_0)
        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_23(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, sign_everything=False, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert signed.has_key("digital_signature")
        sig = signed["digital_signature"]
        assert sig.has_key("signature")
        assert sig["signature"] != None and len(sig["signature"]) > 0

    def test_Sign__0_23__no_passthru(self):
        origJson = json.loads(self.sampleJSON__0_23_0)
        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_23(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, sign_everything=True, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert signed.has_key("digital_signature")
        sig = signed["digital_signature"]
        assert sig.has_key("signature")
        assert sig["signature"] != None and len(sig["signature"]) > 0

    def test_Sign__0_51__no_passthru(self):
        origJson = json.loads(self.sampleJSON__0_51_0)
        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_51(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, sign_everything=True, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(origJson)

        assert signed.has_key("digital_signature")
        sig = signed["digital_signature"]
        assert sig.has_key("signature")
        assert sig["signature"] != None and len(sig["signature"]) > 0

    def testSignUnicode(self):
        if self.testDataUnicode == None:
            log.info("Skipping test, unicode test data file not set.")
            return

        import codecs

        # fileName = "2011-02-28Metadata1004.json"
        # unsigned = json.load(codecs.open(os.path.join(self.testDataDir, fileName), "r", "utf-8-sig"))
        unsigned = json.load(codecs.open(self.testDataUnicode, "r", "utf-8-sig"))

        arbitraryKeyLoc = self.sampleKeyLocations

        signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
        signed = signer.sign(unsigned)

        assert signed.has_key("digital_signature")

        sig = signed["digital_signature"]
        assert sig.has_key("signature")
        assert sig["signature"] != None and len(sig["signature"]) > 0

    def testSignLRTestData(self):
        if self.testDataDir == None:
            log.info("Skipping test, test data directory not set.")
            return

        import codecs

        # allfiles = os.listdir(self.testDataDir)
        for root, dirs, files in os.walk(self.testDataDir):
            for fileName in files:
                log.info("Trying to sign %s" % (fileName, ))

                unsigned = json.load(codecs.open(os.path.join(root, fileName), "r", "utf-8-sig"))

                arbitraryKeyLoc = self.sampleKeyLocations

                signer = Sign_0_21(self.goodkeyid, passphrase=self.goodpassphrase, publicKeyLocations=arbitraryKeyLoc, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin)
                signed = signer.sign(unsigned)

                assert signed.has_key("digital_signature")

                sig = signed["digital_signature"]
                assert sig.has_key("signature")
                assert sig["signature"] != None and len(sig["signature"]) > 0






if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testSigning']
    unittest.main()



'''
Copyright 2011 SRI International

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Created on May 9, 2011

@author: jklo
'''
import unittest, calendar, copy, logging, os
import json
from gnupg import GPG
import time
from LRSignature.sign.Sign import Sign_0_21
from LRSignature.verify.Verify import Verify_0_21
from LRSignature import errors as errors

#logging.basicConfig(level=logging.DEBUG,format="%(asctime)s %(levelname)-5s %(name)-10s %(threadName)-10s %(message)s")

log =  logging.getLogger(__name__)


class Test(unittest.TestCase):
    '''Unit tests for validating signed envelopes from the Learning Registry''' 
    def __init__(self, methodName="runTest"):
        self.sampleJSON = '''
            {
                "_id":"00e3f67232e743b6bc2a079bd98ff55a",
                "_rev":"1-8163d32f6cc9996f2b7228d8b5db7962",
                "doc_type":"resource_data",
                "update_timestamp":"2011-03-14 13:36:04.617999",
                "resource_data":"<oai_dc:dc xmlns:oai_dc=\\"http://www.openarchives.org/OAI/2.0/oai_dc/\\" xmlns:dc=\\"http://purl.org/dc/elements/1.1/\\" xmlns:xsi=\\"http://www.w3.org/2001/XMLSchema-instance\\" xmlns=\\"http://www.openarchives.org/OAI/2.0/\\" xsi:schemaLocation=\\"http://www.openarchives.org/OAI/2.0/oai_dc/                          http://www.openarchives.org/OAI/2.0/oai_dc.xsd\\">\\n<dc:title>A chat about America. October and November, 1884.</dc:title>\\n<dc:creator>J. P.</dc:creator>\\n<dc:subject>United States--Description and travel.</dc:subject>\\n<dc:description>\\"Printed for private circulation only.\\"</dc:description>\\n<dc:description>Electronic reproduction. Washington, D.C. : Library of Congress, [2002-2003]</dc:description>\\n<dc:publisher>Manchester, Palmer &amp; Howe</dc:publisher>\\n<dc:date>1885</dc:date>\\n<dc:type>text</dc:type>\\n<dc:identifier>http://hdl.loc.gov/loc.gdc/lhbtn.12281</dc:identifier>\\n<dc:language>eng</dc:language>\\n<dc:coverage>United States</dc:coverage>\\n</oai_dc:dc>\\n      ",
                "keys":["United States--Description and travel.","eng"],
                "submitter_type":"agent",
                "resource_data_type":"metadata",
                "payload_schema_locator":"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
                "payload_placement":"inline",
                "submitter":"NSDL 2 LR Data Pump",
                "payload_schema":["oai_dc"],
                "node_timestamp":"2011-03-14 13:36:04.617999",
                "doc_version":"0.10.0",
                "create_timestamp":"2011-03-14 13:36:04.617999",
                "active":true,
                "publishing_node":"string",
                "resource_locator":"http://hdl.loc.gov/loc.gdc/lhbtn.12281",
                "doc_ID":"00e3f67232e743b6bc2a079bd98ff55a",
                "TOS": {
                    "submission_TOS": "http://example.com/tos/unknown",
                    "submission_attribution": "unidentified"
                }
            }
            '''
        
        self.sampleKeyLocations = [
                                   "http://example.com/mykey",
                                   "http://example2.com/mykey"
                                   ]
        
        self.gpgbin="gpg"
        self.gnupgHome = os.path.expanduser(os.path.abspath(os.path.join("..", "gnupg_home")))
        self.gpg = None
        
        self.testDataDir = None
        self.testDataUnicode = None

        configFile = os.path.join("config.cfg")
        if os.path.exists(configFile):
            config = json.load(file(configFile))
            
            if config.has_key("global"):
                if config["global"].has_key("testdata") and os.path.exists(config["global"]["testdata"]):
                    self.testDataDir = config["global"]["testdata"]

                if config["global"].has_key("testdata_unicode") and os.path.exists(config["global"]["testdata_unicode"]):
                    self.testDataUnicode = config["global"]["testdata_unicode"]
        
        unittest.TestCase.__init__(self, methodName)


    def setUp(self):
        
        now = time.localtime()
        now = calendar.timegm(now)
        

        try:
            for root, dirs, files in os.walk(self.gnupgHome):
                for filename in files:
                    try:
                        os.unlink(os.path.join(root, filename))
                    except:
                        pass
                os.removedirs( root )
        except:
            pass

        os.makedirs(self.gnupgHome)


        self.gpg = GPG(gpgbinary=self.gpgbin, gnupghome=self.gnupgHome)

        
        self.privateEmail = "privateTest-{0}@learningregistry.org".format(now)
        self.privateEmail2 = "privateTest2-{0}@learningregistry.org".format(now)
        self.genericPassphrase = "supersecret"
        
        input = self.gpg.gen_key_input(name_email=self.privateEmail, passphrase=self.genericPassphrase)
        self.privateKey = self.gpg.gen_key(input)
        
       
        

        input = self.gpg.gen_key_input(name_email=self.privateEmail2, passphrase=self.genericPassphrase)
        self.privateKey2 = self.gpg.gen_key(input)
        
        self.privExport = self.gpg.export_keys([self.privateKey.fingerprint, self.privateKey2.fingerprint], secret=True)
        self.pubExport = self.gpg.export_keys([self.privateKey.fingerprint, self.privateKey2.fingerprint], secret=False)
        
        pass


    def tearDown(self):
        self.gpg.delete_keys([self.privateKey.fingerprint, self.privateKey2.fingerprint], secret=True)
        self.gpg.delete_keys([self.privateKey.fingerprint, self.privateKey2.fingerprint], secret=False)
        pass


    def testGetSignatureBlock(self):
        '''Check that signature block validation correctly returns a structurally valid response'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "envelope did not sign correctly"
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        sigInfo = verifytool._getSignatureInfo(signed)
        
        assert sigInfo != None, "signature extraction from envelope failed"
        
        assert sigInfo.has_key("signing_method") and sigInfo["signing_method"] == verifytool.signatureMethod, "signing_method is missing from signature block"
        
        assert sigInfo.has_key("signature") and sigInfo["signature"] != None and len(sigInfo["signature"]) > 0, "signature field is missing, null or is empty"
        
        assert sigInfo.has_key("key_location") and sigInfo["key_location"] == self.sampleKeyLocations, "key_location field is not correct"
        
        assert sigInfo.has_key("key_owner") and sigInfo["key_owner"] == signtool._get_privatekey_owner(), "key_owner field does not match signing key"

    def testBadSignatureBlockMissingLocation(self):
        '''Check that signature block validation correctly checks for missing key_location field'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=None)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope not signed correctly"
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        with self.assertRaises(errors.BadSignatureFormat, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        
        assert caught.exception.message == errors.MISSING_KEY_LOCATION, "Exception not formatted with correct message" 
        
        
    def testBadSignatureBlockMissingSignatureMethod(self):
        '''Check signature block for missing signature_method detection'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope not signed correctly."
        
        del signed["digital_signature"]["signing_method"]
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        with self.assertRaises(errors.UnsupportedSignatureAlgorithm, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        
        assert caught.exception.alg == None, "Raised exception not formatted correctly" 
        
        
    def testBadSignatureBlockBadSignatureMethod(self):
        '''Check signature block for detecting unsupported algorithm use'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None
        
        signed["digital_signature"]["signing_method"] = signtool.signatureMethod+"+BAD_SIGNATURE_METHOD"
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        with self.assertRaises(errors.UnsupportedSignatureAlgorithm, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        
        assert caught.exception.alg == signed["digital_signature"]["signing_method"], "Exception not raised correctly."
        
    def testBadSignatureBlockBadKeyOwner(self):
        '''Check signature block for a bad key_owner field'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope not signed correctly"
        
        signed["digital_signature"]["key_owner"] = ["John Q. Public <johnqpublic@learningregistry.org>"]
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        with self.assertRaises(errors.BadSignatureFormat, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        
        assert caught.exception.message == errors.BAD_KEY_OWNER, "Wrong exception"
        
        
    def testBadSignatureBlockMissingNullEmptySignature(self):
        '''Check signature block validation with missing/null or empty signature'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope signing did not succeed correctly."
                
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        signed["digital_signature"]["signature"] = ""
        with self.assertRaises(errors.BadSignatureFormat, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        assert caught.exception.message == errors.MISSING_SIGNATURE, "Wrong exception raised"
        
        signed["digital_signature"]["signature"] = None
        with self.assertRaises(errors.BadSignatureFormat, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        assert caught.exception.message == errors.MISSING_SIGNATURE, "Wrong exception raised"
        
        del signed["digital_signature"]["signature"] 
        with self.assertRaises(errors.BadSignatureFormat, msg="Expected exception not raised.") as caught:
            sigInfo = verifytool._getSignatureInfo(signed)
        assert caught.exception.message == errors.MISSING_SIGNATURE, "Wrong exception raised"
    
    
    def testValidSignature(self):
        '''Check for valid signature'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope did not sign correctly"
                
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        verified = verifytool.verify(signed)
        assert verified == True, "Envelope signature verification did not succeed, even though it should"
        
    def testMissingPublicKey(self):
        '''Check for appropriate response when public key for signature is missing'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "Envelope did not sign correctly"
        
        self.gpg.delete_keys([self.privateKey.fingerprint], secret=True)
        self.gpg.delete_keys([self.privateKey.fingerprint], secret=False)
                
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        with self.assertRaises(errors.MissingPublicKey, msg="Expected exception not raised.") as caught:
            verified = verifytool.verify(signed)
            assert verified == False, "Envelope verified, despite missing public key."
    
    def testCorruptEnvelope(self):
        '''Modify a signed envelope and check for validity'''
        unsigned = json.loads(self.sampleJSON)
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "envelope did not get signed correctly"
        
        signed["X_corrupted"] = "Corrupted Envelope"
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)

        verified = verifytool.verify(signed)
        
        assert verified == False, "corrupted envelope verified as good"
        
        
    def testWrongSignature(self):
        '''Test using a mis-matched signature, using a signature from a different valid envelope'''
        unsigned = json.loads(self.sampleJSON)
        altered = copy.deepcopy(unsigned)
        altered["X_corrupted"] = "Altered Envelope"
        
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signtool2 = Sign_0_21(privateKeyID=self.privateKey2.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        
        signed = signtool.sign(unsigned)
        alt_signed = signtool2.sign(altered)
        
        assert signed != None, "original did not get signed"
        assert alt_signed != None, "modified copy did not get signed"
        
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        
        verified = verifytool.verify(signed)
        assert verified == True, "signature did not verify, even though it should."
        verified = verifytool.verify(alt_signed)
        assert verified == True, "signature did not verify, even though it should."
        
        signed["digital_signature"] = alt_signed["digital_signature"]
        
        verified = verifytool.verify(signed)
        assert verified == False, "swapped signature block validated envelope as good."
        
    def testCorruptSignature(self):
        '''Test using a corrupted signature, replace the hash within a signature with a hash from a different envelope'''
        unsigned = json.loads(self.sampleJSON)
        
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        signed = signtool.sign(unsigned)
        assert signed != None, "baseline signing failed"
        
        # validate original with good signature
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)
        verified = verifytool.verify(signed)
        assert verified == True, "baseline validation failed"
        
        # manipulate the hash portion of a signature block
        altered = copy.deepcopy(unsigned)
        altered["X_corrupted"] = "Altered Envelope"
        altered_hash = signtool.get_message(altered)
        
        validHash = verifytool._extractHashFromSignature(signed["digital_signature"]["signature"])
        alt_signed = copy.deepcopy(signed)
        
        alt_signed["digital_signature"]["signature"] = signed["digital_signature"]["signature"].replace(validHash, altered_hash)
        assert alt_signed["digital_signature"]["signature"] != signed["digital_signature"]["signature"], "envelopes should not be equal after deliberate modificaton"
        
        verified = verifytool.verify(alt_signed)
        assert verified == False, "verification failed, corrupted signature block verified as good"
        
    def testSignLRTestData(self):
        '''Test using LR Test Data, if available'''
        if self.testDataDir == None:
            log.info("Skipping test, test data directory not set.")
            return
        
        import codecs
        
        signtool = Sign_0_21(privateKeyID=self.privateKey.fingerprint, passphrase=self.genericPassphrase, gnupgHome=self.gnupgHome, gpgbin=self.gpgbin, publicKeyLocations=self.sampleKeyLocations)
        verifytool = Verify_0_21(gpgbin=self.gpgbin, gnupgHome=self.gnupgHome)

        allfiles = os.listdir(self.testDataDir)
        for root, dirs, files in os.walk(self.testDataDir):

            for fileName in files:
                log.info("Trying to sign %s" % (fileName, ))
                
                unsigned = json.load(codecs.open(os.path.join(root, fileName), "r", "utf-8-sig"))
                
                signed = signtool.sign(unsigned)
                
                assert signed.has_key("digital_signature"), "missing digital_signature"
                
                verified = verifytool.verify(signed)
                assert verified == True, "baseline validation failed"
        
        
        
            
        
    


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testGetSignatureBlock']
    unittest.main()